@@ -725,47 +725,84 @@ implements Comparable<Decimal64>, java.io.Serializable {
 	 * @param l
 	 * @return
 	 */
-	public static String toString(long l) {
-		StringBuilder b;
-		BCDDecimal d;
-		int e, i;
-		boolean bb = false;
-
-		if(isNaN(l)) {
+//	public static String toString(long l) {
+//		StringBuilder b;
+//		BCDDecimal d;
+//		int e, i;
+//		boolean bb = false;
+//
+//		if(isNaN(l)) {
+//			return "NaN";
+//		} else if(isInfinite(l)) {
+//			return getSignum(l) > 0 ? "Infinity" : "-Infinity";
+//		} else if(isZero(l)) {
+//			return getSignum(l) > 0 ? "0.0" : "-0.0";
+//		} else {
+//			b = new StringBuilder();
+//			e = getExponent(l);
+//			d = getFractionField(l);
+//			if(getSignum(l) < 0) {
+//				b.append('-');
+//			}
+//
+//			if(e > 10 || e < -4) {
+//				b.append(d.toString());
+//				b.append("e").append(e);
+//			} else {
+//				i = 15;
+//				for(; i - e >= 0; i--) {
+//					if(bb || d.getDigit(i - e) > 0) {
+//						b.append((char)(d.getDigit(i - e) + '0'));
+//						bb = true;
+//					}
+//				}
+//				b.append('.');
+//				if(i < 0) {
+//					b.append('0');
+//				} else {
+//					for(; i >= 0; i--) {
+//						b.append((char)(d.getDigit(i - e) + '0'));
+//					}
+//				}
+//			}
+//			return b.toString();
+//		}
+//	}
+	public static String toString(long value) {
+		if(isNaN(value)) {
 			return "NaN";
-		} else if(isInfinite(l)) {
-			return getSignum(l) > 0 ? "Infinity" : "-Infinity";
-		} else if(isZero(l)) {
-			return getSignum(l) > 0 ? "0.0" : "-0.0";
+		} else if(isInfinite(value)) {
+			return getSignum(value) > 0 ? "Infinity" : "-Infinity";
+		} else if(isZero(value)) {
+			return getSignum(value) > 0 ? "0.0" : "-0.0";
 		} else {
-			b = new StringBuilder();
-			e = getExponent(l);
-			d = getFractionField(l);
-			if(getSignum(l) < 0) {
-				b.append('-');
-			}
+			BCDDecimal f = getFractionField(value);
+			int e = getExponent(value);
 
-			if(e > 10 || e < -4) {
-				b.append(d.toString());
-				b.append("e").append(e);
-			} else {
-				i = 15;
-				for(; i - e >= 0; i--) {
-					if(bb || d.getDigit(i - e) > 0) {
-						b.append((char)(d.getDigit(i - e) + '0'));
-						bb = true;
-					}
-				}
-				b.append('.');
-				if(i < 0) {
-					b.append('0');
-				} else {
-					for(; i >= 0; i--) {
-						b.append((char)(d.getDigit(i - e) + '0'));
-					}
-				}
-			}
-			return b.toString();
+			return f.shift(e).toString()
+					.replaceFirst("0+((e(\\+|-)?[0-9]+)?)$", "$1");
+		}
+	}
+
+	/**
+	 * 
+	 * @param value
+	 * @param precision
+	 * @return
+	 */
+	public static String toString(long value, int precision) {
+		if(isNaN(value)) {
+			return "NaN";
+		} else if(isInfinite(value)) {
+			return getSignum(value) > 0 ? "Infinity" : "-Infinity";
+		} else if(isZero(value)) {
+			return getSignum(value) > 0 ? "0.0" : "-0.0";
+		} else {
+			BCDDecimal f = getFractionField(value);
+			int e = getExponent(value);
+
+			return f.shift(e).toString(precision)
+					.replaceFirst("0+((e(\\+|-)?[0-9]+)?)$", "$1");
 		}
 	}
 
