@@ -21,6 +21,7 @@ import java.util.List;
 import java.util.Map;
 
 import net.morilib.lisp.CompiledCode.Builder;
+import net.morilib.lisp.chihaya.LispSyntax;
 import net.morilib.lisp.subr.BinaryArgs;
 import net.morilib.lisp.subr.UnaryArgs;
 
@@ -222,6 +223,7 @@ import net.morilib.lisp.subr.UnaryArgs;
 			List<Cons> symlist,
 			CodeExecutor exec,
 			IntStack memento,
+			boolean toplv,
 			LispCompiler.MiscInfo syncased) {
 		//int pos = -1;
 
@@ -238,7 +240,13 @@ import net.morilib.lisp.subr.UnaryArgs;
 			compileArgsBind(bcdr, env, builder, s2, sl);
 			builder.addRewindCont(pos);*/
 		} else {
-			compile(bcar, env, builder, false, symcall, false, symlist,
+			boolean tlv;
+
+			tlv = (toplv && bcar instanceof Cons &&
+					((Cons)bcar).getCar() instanceof SymbolName &&
+					((SymbolName)((Cons)bcar).getCar())
+					.getName().equals(SynBegin.BEGIN.getName()));
+			compile(bcar, env, builder, tlv, symcall, false, symlist,
 					exec, memento, syncased);
 			compileArgs(
 					bcdr, env, builder, symcall, symlist,
@@ -293,6 +301,11 @@ import net.morilib.lisp.subr.UnaryArgs;
 
 		if(bcar instanceof Symbol) {
 			ddd = env.findDatum((Symbol)bcar);
+//			if(ddd == null && ((Symbol)bcar).getNamespace() != null) {
+//				ddd = env.findDatum(
+//						((Symbol)bcar).getNamespace()
+//						.getSymbol(((Symbol)bcar).getName()));
+//			}
 		} else {
 			SymbolScope ss   = (SymbolScope)bcar;
 			Environment menv =
@@ -301,6 +314,11 @@ import net.morilib.lisp.subr.UnaryArgs;
 			//System.out.println(ss);
 			//System.out.println(menv);
 			ddd = menv.findDatum(ss.getSymbol());
+//			if(ddd == null && ss.getSymbol().getNamespace() != null) {
+//				ddd = env.findDatum(
+//						ss.getSymbol().getNamespace()
+//						.getSymbol(ss.getSymbol().getName()));
+//			}
 		}
 		return ddd;
 	}
@@ -368,6 +386,10 @@ import net.morilib.lisp.subr.UnaryArgs;
 			if(bcar instanceof Symbol || bcar instanceof SymbolScope) {
 				Datum ddd = getSym(bcar, env);
 
+				if(ddd instanceof LispLazySubrSyn) {
+					ddd = ((LispLazySubrSyn)ddd).setup();
+				}
+
 				if(ddd instanceof SynSetS) {
 					if(bcdr instanceof Cons) {
 						Datum d4 = ((Cons)bcdr).getCar();
@@ -473,11 +495,16 @@ import net.morilib.lisp.subr.UnaryArgs;
 					compile(exp, env, builder, toplevel,
 							symcall, istail, symlist,
 							exec, memento, syncased);
+				} else if(ddd instanceof MacroQuote) {
+					// traditional macro
+					Datum exp = expandMacro(body, env, exec, memento);
+
+					builder.addPush(exp);
 				} else {
 					compileSexp(
 							bcar, bcdr, env, builder,
 							symcall, istail, symlist,
-							exec, memento, syncased);
+							exec, memento, toplevel, syncased);
 				}
 			} else if(bcar instanceof SynSyntax ||
 					bcar instanceof SynQuasisyntax) {
@@ -495,8 +522,10 @@ import net.morilib.lisp.subr.UnaryArgs;
 				compileSexp(
 						bcar, bcdr, env, builder,
 						symcall, istail, symlist,
-						exec, memento, syncased);
+						exec, memento, toplevel, syncased);
 			}
+		} else if(body instanceof LispSyntax) {
+			builder.addPush(((LispSyntax)body).getWrapped());
 		} else {
 			builder.addPush(body);
 		}
@@ -509,13 +538,17 @@ import net.morilib.lisp.subr.UnaryArgs;
 			Datum bcdr,
 			Environment env,
 			Environment ienv,
+			boolean toplv,
 			int ttype) {
 		Cons res = new Cons();
+		boolean tlv;
 
-		res.setCar(replaceLocalVals(bcar, env, ienv, false, ttype));
-
+		tlv = (toplv && bcar instanceof Cons &&
+				(!(((Cons)bcar).getCar() instanceof SymbolName) ||
+						((SymbolName)((Cons)bcar).getCar())
+						.getName().equals(SynBegin.BEGIN.getName())));
+		res.setCar(replaceLocalVals(bcar, env, ienv, tlv, ttype));
 		res.setCdr(replaceLocalValsArgs(bcdr, env, ienv, ttype));
-
 		return res;
 	}
 
@@ -658,6 +691,10 @@ import net.morilib.lisp.subr.UnaryArgs;
 				Symbol sym = ((SymbolName)bcar).getSymbol();
 				Datum  d2  = getSym(bcar, env);
 
+				if(d2 instanceof LispLazySubrSyn) {
+					d2 = ((LispLazySubrSyn)d2).setup();
+				}
+
 				if(d2 instanceof SynSetS) {
 					if(bcdr instanceof Cons) {
 						Datum d4 = ((Cons)bcdr).getCar();
@@ -708,18 +745,18 @@ import net.morilib.lisp.subr.UnaryArgs;
 
 					if(usyn.getLambda() == null) {
 						exp = replaceLocalValsSexp(
-								bcar, bcdr, env, ienv, ttype);
+								bcar, bcdr, env, ienv, toplv, ttype);
 					} else {
 						exp = body;
 					}
 					return exp;
 				} else {
 					return replaceLocalValsSexp(
-							bcar, bcdr, env, ienv, ttype);
+							bcar, bcdr, env, ienv, toplv, ttype);
 				}
 			} else {
 				return replaceLocalValsSexp(
-						bcar, bcdr, env, ienv, ttype);
+						bcar, bcdr, env, ienv, toplv, ttype);
 			}
 		} else {
 			// 束縛変数はない
@@ -809,13 +846,58 @@ import net.morilib.lisp.subr.UnaryArgs;
 							cl.getParameterList(),
 							c.getCdr(), nenv, message);
 					res.d = exec.exec(cl.getCode(), nenv, memento);
+					if(res.d instanceof LispSyntax) {
+						res.d = ((LispSyntax)res.d).getWrapped();
+					}
+//					res.d = MacroUtils72.unquote(res.d,
+//							new EnvironmentObject(nenv, false));
 					res.dirty = true;
 					return res;
-				} else if(d instanceof SynQuote) {
+				} else if(d instanceof MacroQuote) {
+					// macro expansion
+					MacroQuote m  = (MacroQuote)d;
+					Closure    cl = m.getClosure();
+					Environment nenv = new Environment(env);
+					//Object memento = exec.newMemento();
+
+					IntLispUtils.bindLocal(
+							cl.getParameterList(),
+							c.getCdr(), nenv, message);
+					res.d = exec.exec(cl.getCode(), nenv, memento);
+//					if(res.d instanceof LispSyntax) {
+//						res.d = ((LispSyntax)res.d).getWrapped();
+//					}
+					res.dirty = true;
+					return res;
+				} else if(d instanceof Macro72) {
+					// macro expansion
+					Macro72 m  = (Macro72)d;
+					Closure cl = m.getClosure();
+					Environment nenv = new Environment(env);
+
+					IntLispUtils.bindLocal(
+							cl.getParameterList(),
+							new Cons(c, Nil.NIL), nenv, message);
+					res.d = exec.exec(cl.getCode(), nenv, memento);
+					if(res.d instanceof LispSyntax) {
+						res.d = ((LispSyntax)res.d).getWrapped();
+					} else {
+//						throw message.getError(
+//								"err.require.syntaxobject", res.d);
+					}
+					res.dirty = true;
+					return res;
+				} else if(d instanceof SynQuote ||
+						d instanceof SynQuasiquote) {
 					// quote: not expand macro
 					res.d = body;
 					res.dirty = false;
 					return res;
+//				} else if(d instanceof LispSyntax) {
+//					// quote: not expand macro
+//					res.d = body;
+//					res.dirty = false;
+//					return res;
 				} else {
 					return expandMacroSexp(
 							c.getCar(), c.getCdr(), env,
@@ -826,6 +908,12 @@ import net.morilib.lisp.subr.UnaryArgs;
 						c.getCar(), c.getCdr(), env,
 						exec, memento);
 			}
+//		} else if(body instanceof LispSyntax) {
+////			return expandMacro1i(((LispSyntax)body).getWrapped(),
+////					env, exec, memento);
+//			res.d = ((LispSyntax)body).getWrapped();
+//			res.dirty = false;
+//			return res;
 		} else {
 			res.d = body;
 			res.dirty = false;
@@ -852,7 +940,8 @@ import net.morilib.lisp.subr.UnaryArgs;
 			Environment env,
 			CodeExecutor exec,
 			IntStack memento) {
-		D2 loop = new D2(body, false);
+		Datum d = LispFeature.evalSRFI7(body, env, message);
+		D2 loop = new D2(d, false);
 
 		// 展開するマクロがなくなるまでループ
 		do {
@@ -879,10 +968,10 @@ import net.morilib.lisp.subr.UnaryArgs;
 				}*/
 			}
 
-			// マクロ展開
+			// macro expansion
 			loop = expandMacro1i(b, env, exec, memento);
 		} while(loop.dirty);
-
+//		System.out.println(LispUtils.print(loop.d));
 		return loop.d;
 	}
 
