@@ -177,7 +177,7 @@ public final class LispMath {
 		} else if(nm1.isInteger()) {
 			LispReal rm2 = nm2.getReal();
 			BigInteger b = rm2.getBigInteger();
-			Integer b2 = IntLispUtils.toIntExact(b);
+			Integer b2 = LispUtils.toIntExact(b);
 			BigInteger a = nm1.getReal().getBigInteger();
 
 			if(b2 == null) {
@@ -195,7 +195,7 @@ public final class LispMath {
 		} else if(nm1.isRational()) {
 			LispReal rm2 = nm2.getReal();
 			BigInteger b = rm2.getBigInteger();
-			Integer b2 = IntLispUtils.toIntExact(b);
+			Integer b2 = LispUtils.toIntExact(b);
 			BigInteger an = nm1.getReal().getNumerator();
 			BigInteger ad = nm1.getReal().getDenominator();
 
