@@ -18,6 +18,12 @@ package net.morilib.lisp;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
+import net.morilib.lang.Decimal32;
+import net.morilib.lang.DoubleUtils;
+import net.morilib.lisp.math.ILispExactOrInexactQuantity;
+import net.morilib.lisp.math.algebra.ILispField;
+import net.morilib.lisp.math.algebra.ILispNorm;
+import net.morilib.lisp.math.algebra.ILispNumberEqual;
 import net.morilib.lisp.sos.LispType;
 
 /**
@@ -26,7 +32,16 @@ import net.morilib.lisp.sos.LispType;
  *
  * @author MORIGUCHI, Yuichiro 2009
  */
-public abstract class LispNumber extends Atom {
+public abstract class LispNumber extends Atom
+implements ILispField<LispNumber>, ILispNorm,
+ILispExactOrInexactQuantity<LispNumber>,
+ILispNumberEqual<LispNumber> {
+
+	//
+	private static final int I_BIT = 0x1;
+	//private static final int J_BIT = 0x2;
+	//private static final int K_BIT = 0x3;
+	private static final int L_BIT = 0x4;
 
 	/*package*/ static String disp(double n) {
 		if(n == Double.POSITIVE_INFINITY) {
@@ -109,6 +124,22 @@ public abstract class LispNumber extends Atom {
 		}
 	}
 
+	//
+	private static boolean isImaginary(int r) {
+		return r == 'i' || r == 'j' || r == 'k' || r == 'o';
+	}
+
+	//
+	private static int getImaginaryNo(int r) {
+		switch(r) {
+		case 'i':  return 1;
+		case 'j':  return 2;
+		case 'k':  return 3;
+		case 'o':  return 4;
+		default:   throw new RuntimeException();
+		}
+	}
+
 	private static int read(String s, int p) {
 		return (p >= s.length()) ? -1 : s.charAt(p);
 	}
@@ -199,75 +230,167 @@ public abstract class LispNumber extends Atom {
 	}
 
 	private static LispNumber getImaginary(
-			String gtok1, int radix, Exact exact) {
-		// 虚部
-		double i;
-		String   iii = gtok1.replaceFirst("i$", "");
+			String gtok1, int rx, Exact ex, int imgn) {
+		String iii = gtok1.replaceFirst("[ijko][ijk]?$", "");
+		LispReal im;
 
-		if(radix != 10) {
-			return null;
-		} else if(Exact.EXACT.equals(exact)) {
+		if(rx != 10) {
 			return null;
 		}
 
+		// imaginary part
 		if("+".equals(iii)) {
-			i = 1.0;
+			im = !Exact.INEXACT.equals(ex) ?
+					LispInteger.ONE : new LispDouble(1.0);
 		} else if("-".equals(iii)) {
-			i = -1.0;
+			im = !Exact.INEXACT.equals(ex) ?
+					LispInteger.ONE.uminus() : new LispDouble(-1.0);
+		} else if(iii.indexOf('/') >= 0) {
+			BigInteger num, den;
+			String[] nd = iii.split("/");
+
+			try {
+				num = getValueByBigInt(nd[0], rx);
+				den = getValueByBigInt(nd[1], rx);
+				if(Exact.INEXACT.equals(ex)) {
+					double numd = num.doubleValue();
+					double dend = den.doubleValue();
+
+					im = new LispDouble(numd / dend);
+				} else {
+					im = LispRational.newRational(num, den);
+				}
+			} catch(NumberFormatException e) {
+				return null;
+			}
+		} else if(!Exact.INEXACT.equals(ex) &&
+				iii.matches("^[-+]?[0-9]+$")) {
+			iii = iii.replaceFirst("^\\+", "");
+			im = LispInteger.valueOf(getValueByBigInt(iii, rx));
 		} else {
-			i = getValueByDouble(iii);
+			im = new LispDouble(getValueByDouble(iii));
 		}
 
-		return LispComplex.newComplex(0.0, i);
+		switch(imgn) {
+		case 1:
+			return LispComplex.newComplex(LispInteger.ZERO, im);
+		case 2:
+			return LispQuaternion.newQuaternion(
+					LispInteger.ZERO, LispInteger.ZERO,
+					im, LispInteger.ZERO);
+		case 3:
+			return LispQuaternion.newQuaternion(
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, im);
+		case 4:
+			return LispOctonion.newOctonion(
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO,
+					im, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO);
+		case 5:
+			return LispOctonion.newOctonion(
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, im,
+					LispInteger.ZERO, LispInteger.ZERO);
+		case 6:
+			return LispOctonion.newOctonion(
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO,
+					im, LispInteger.ZERO);
+		case 7:
+			return LispOctonion.newOctonion(
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, LispInteger.ZERO,
+					LispInteger.ZERO, im);
+		default:
+			throw new RuntimeException("" + imgn);
+		}
 	}
 
 	private static LispNumber getComplex(
-			String gtok1, String gtok2, int radix, Exact exact) {
-		// 虚部
-		double i;
-		String   iii = gtok2.replaceFirst("i$", "");
+			String gtok1, String[] bufi, int rx, Exact ex) {
+		LispReal re;
+		LispReal[] im = new LispReal[8];
+		String iii;
 
-		if(radix != 10) {
-			return null;
-		} else if(Exact.EXACT.equals(exact)) {
+		if(rx > 0 && rx != 10) {
 			return null;
 		}
 
-		if("+".equals(iii)) {
-			i = 1.0;
-		} else if("-".equals(iii)) {
-			i = -1.0;
+		// real part
+		if(gtok1.indexOf('/') >= 0) {
+			BigInteger num, den;
+			String[] nd = gtok1.split("/");
+
+			try {
+				num = getValueByBigInt(nd[0], rx);
+				den = getValueByBigInt(nd[1], rx);
+				if(Exact.INEXACT.equals(ex)) {
+					double numd = num.doubleValue();
+					double dend = den.doubleValue();
+
+					re = new LispDouble(numd / dend);
+				} else {
+					re = LispRational.newRational(num, den);
+				}
+			} catch(NumberFormatException e) {
+				return null;
+			}
+		} else if(!Exact.INEXACT.equals(ex) &&
+				gtok1.matches("^[-+]?[0-9]+$")) {
+			re = LispInteger.valueOf(getValueByBigInt(gtok1, rx));
 		} else {
-			i = getValueByDouble(iii);
+			re = new LispDouble(getValueByDouble(gtok1));
 		}
 
-		if(i != 0.0) {
-			// 実部
-			double r = getValueByDouble(gtok1);
+		// imaginary part
+		for(int i = 1; i < 8; i++) {
+			if(bufi[i] == null) {
+				im[i] = LispInteger.ZERO;
+				continue;
+			}
 
-			return LispComplex.newComplex(r, i);
-		} else {
-			if(gtok1.indexOf('/') > 0) {
-				// 有理数
-				String[] nd = gtok1.split("/");
-				BigInteger num = new BigInteger(nd[0]);
-				BigInteger den = new BigInteger(nd[1]);
-
-				return LispRational.newRational(num, den);
-			} else if(gtok1.matches("^[0-9]+$")) {
-				// 整数
-				BigInteger  val  = new BigInteger(gtok1);
-				LispInteger atom = LispInteger.valueOf(val);
-
-				return atom;
+			iii = bufi[i].toString().replaceFirst("[ijko][ijk]?$", "");
+			if("+".equals(iii)) {
+				im[i] = !Exact.INEXACT.equals(ex) ?
+						LispInteger.ONE : new LispDouble(1.0);
+			} else if("-".equals(iii)) {
+				im[i] = !Exact.INEXACT.equals(ex) ?
+						LispInteger.ONE.uminus() : new LispDouble(-1.0);
+			} else if(iii.indexOf('/') >= 0) {
+				BigInteger num, den;
+				String[] nd = iii.split("/");
+	
+				try {
+					num = getValueByBigInt(nd[0], rx);
+					den = getValueByBigInt(nd[1], rx);
+					if(Exact.INEXACT.equals(ex)) {
+						double numd = num.doubleValue();
+						double dend = den.doubleValue();
+	
+						im[i] = new LispDouble(numd / dend);
+					} else {
+						im[i] = LispRational.newRational(num, den);
+					}
+				} catch(NumberFormatException e) {
+					return null;
+				}
+			} else if(!Exact.INEXACT.equals(ex) &&
+					iii.matches("^[-+]?[0-9]+$")) {
+				iii = iii.replaceFirst("^\\+", "");
+				im[i] = LispInteger.valueOf(getValueByBigInt(iii, rx));
 			} else {
-				// 浮動小数点数
-				double val = parseDouble(gtok1);
-				LispDouble atom = new LispDouble(val);
-
-				return atom;
+				im[i] = new LispDouble(getValueByDouble(iii));
 			}
 		}
+
+		return LispOctonion.newOctonion(
+				re,    im[1], im[2], im[3],
+				im[4], im[5], im[6], im[7]);
 	}
 
 	private static LispNumber getPolar(
@@ -288,8 +411,7 @@ public abstract class LispNumber extends Atom {
 			double a = getValueByDouble(ra[1]);
 
 			return LispComplex.newComplex(
-					r * Math.cos(a),
-					r * Math.sin(a));
+					r * Math.cos(a), r * Math.sin(a));
 		}
 	}
 
@@ -302,12 +424,14 @@ public abstract class LispNumber extends Atom {
 	public static LispNumber parse(String s, int radix) {
 		StringBuilder b1 = new StringBuilder();
 		StringBuilder b2 = new StringBuilder();
+		String[] bi = new String[8];
 		State state = State.INIT;
 		boolean radix2 = false;
 		Exact exact = Exact.DEFAULT;
 		boolean signed = false;
 		int p = 0;
 		int r = read(s, p);
+		int r0, imgn;
 
 		if(radix < 1 || radix > 16) {
 			throw new IllegalArgumentException("wrong radix");
@@ -396,7 +520,7 @@ public abstract class LispNumber extends Atom {
 					b1.append((char)r);
 					state = State.FLOAT_NUMBER;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b1.append((char)r);
 					state = State.COMPLEX_I;
 					r = read(s, ++p);
@@ -415,9 +539,12 @@ public abstract class LispNumber extends Atom {
 					r = read(s, ++p);
 				} else if(r == 'i') {
 					b1.append((char)r);
-					//state = State.COMPLEX_I;
 					state = State.INF_I;
 					r = read(s, ++p);
+				} else if(isImaginary(r)) {
+					b1.append((char)r);
+					state = State.COMPLEX_I;
+					r = read(s, ++p);
 				} else if(r == 'n') {
 					b1.append((char)r);
 					state = State.NAN_N1;
@@ -452,7 +579,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.HEAD_SIGN_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b1.append((char)r);
 					state = State.COMPLEX_I;
 					r = read(s, ++p);
@@ -487,7 +614,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.HEAD_SIGN_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b1.append((char)r);
 					state = State.COMPLEX_I;
 					r = read(s, ++p);
@@ -531,7 +658,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.HEAD_SIGN_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b1.append((char)r);
 					state = State.COMPLEX_I;
 					r = read(s, ++p);
@@ -564,7 +691,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.HEAD_SIGN_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b1.append((char)r);
 					state = State.COMPLEX_I;
 					r = read(s, ++p);
@@ -709,12 +836,25 @@ public abstract class LispNumber extends Atom {
 				}
 				break;
 			case INF_I:
+				imgn = I_BIT;
+				if(r == 'o') {
+					imgn = imgn | L_BIT;
+					r = read(s, ++p);
+				}
+
 				if(r < 0) {
-					return getImaginary(b1.toString(), radix, exact);
+					return getImaginary(
+							b1.toString(), radix, exact, imgn);
 				} else if(r == 'n') {
 					b1.append((char)r);
 					state = State.INF_N;
 					r = read(s, ++p);
+				} else if(r == '+' || r == '-') {
+					bi[I_BIT] = b1.toString();
+					b1 = new StringBuilder("0");
+					b2.append((char)r);
+					state = State.HEAD_SIGN_COMPLEX;
+					r = read(s, ++p);
 				} else {
 					return null;
 				}
@@ -759,7 +899,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.HEAD_SIGN_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b1.append((char)r);
 					state = State.COMPLEX_I;
 					r = read(s, ++p);
@@ -815,13 +955,31 @@ public abstract class LispNumber extends Atom {
 				}
 				//break;
 			case COMPLEX_I:
+				r0   = b1.charAt(b1.length() - 1);
+				imgn = getImaginaryNo(r0);
+				if(r == 'o') {
+					if(r0 == 'o') {
+						return null;
+					} else {
+						imgn = imgn | L_BIT;
+						r = read(s, ++p);
+					}
+				}
+				bi[imgn] = b1.toString();
+
 				if(r < 0) {
 					if(signed) {
 						return getImaginary(
-								b1.toString(), radix, exact);
+								b1.toString(), radix, exact, imgn);
 					} else {
 						return null;
 					}
+				} else if(r == '+' || r == '-') {
+					bi[imgn] = b1.toString();
+					b1 = new StringBuilder("0");
+					b2.append((char)r);
+					state = State.HEAD_SIGN_COMPLEX;
+					r = read(s, ++p);
 				} else {
 					return null;
 				}
@@ -834,9 +992,12 @@ public abstract class LispNumber extends Atom {
 					r = read(s, ++p);
 				} else if(r == 'i') {
 					b2.append((char)r);
-					//state = State.COMPLEX_I2;
 					state = State.INF_I_COMPLEX;
 					r = read(s, ++p);
+				} else if(isImaginary(r)) {
+					b2.append((char)r);
+					state = State.COMPLEX_I2;
+					r = read(s, ++p);
 				} else if(r < 0) {
 					return null;
 				} else {
@@ -859,7 +1020,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.RATIONAL_NUMBER_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b2.append((char)r);
 					state = State.COMPLEX_I2;
 					r = read(s, ++p);
@@ -886,7 +1047,7 @@ public abstract class LispNumber extends Atom {
 					b2.append((char)r);
 					state = State.FLOAT_NUMBER_E_COMPLEX;
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b2.append((char)r);
 					state = State.COMPLEX_I2;
 					r = read(s, ++p);
@@ -922,7 +1083,7 @@ public abstract class LispNumber extends Atom {
 				if(isAlnum(r, radix)) {
 					b2.append((char)r);
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b2.append((char)r);
 					state = State.COMPLEX_I2;
 					r = read(s, ++p);
@@ -947,7 +1108,7 @@ public abstract class LispNumber extends Atom {
 				if(isAlnum(r, radix)) {
 					b2.append((char)r);
 					r = read(s, ++p);
-				} else if(r == 'i') {
+				} else if(isImaginary(r)) {
 					b2.append((char)r);
 					state = State.COMPLEX_I2;
 					r = read(s, ++p);
@@ -958,13 +1119,29 @@ public abstract class LispNumber extends Atom {
 				}
 				break;
 			case INF_I_COMPLEX:
+				imgn = I_BIT;
+				if(r == 'o') {
+					imgn = imgn | L_BIT;
+					r = read(s, ++p);
+				}
+
+				if(bi[imgn] != null) {
+					return null;
+				}
+
 				if(r < 0) {
-					return getComplex(
-							b1.toString(), b2.toString(), radix, exact);
+					bi[imgn] = b2.toString();
+					return getComplex(b1.toString(), bi, radix, exact);
 				} else if(r == 'n') {
 					b2.append((char)r);
 					state = State.INF_N_COMPLEX;
 					r = read(s, ++p);
+				} else if(r == '+' || r == '-') {
+					bi[imgn] = b2.toString();
+					b2 = new StringBuilder();
+					b2.append((char)r);
+					state = State.HEAD_SIGN_COMPLEX;
+					r = read(s, ++p);
 				} else {
 					return null;
 				}
@@ -997,7 +1174,7 @@ public abstract class LispNumber extends Atom {
 				}
 				break;
 			case INF_0_COMPLEX:
-				if(r == 'i') {
+				if(isImaginary(r)) {
 					b2.append((char)r);
 					state = State.COMPLEX_I2;
 					r = read(s, ++p);
@@ -1006,9 +1183,29 @@ public abstract class LispNumber extends Atom {
 				}
 				break;
 			case COMPLEX_I2:
+				r0   = b2.charAt(b2.length() - 1);
+				imgn = getImaginaryNo(r0);
+				if(r == 'o') {
+					if(r0 == 'o') {
+						return null;
+					} else {
+						imgn = imgn | L_BIT;
+						r = read(s, ++p);
+					}
+				}
+
+				if(bi[imgn] != null) {
+					return null;
+				}
+				bi[imgn] = b2.toString();
+
 				if(r < 0) {
-					return getComplex(
-							b1.toString(), b2.toString(), radix, exact);
+					return getComplex(b1.toString(), bi, radix, exact);
+				} else if(r == '+' || r == '-') {
+					b2 = new StringBuilder();
+					b2.append((char)r);
+					state = State.HEAD_SIGN_COMPLEX;
+					r = read(s, ++p);
 				} else {
 					return null;
 				}
@@ -1100,6 +1297,26 @@ public abstract class LispNumber extends Atom {
 	 */
 	public abstract LispString toLispString(int radix);
 
+	/**
+	 * 
+	 * @param radix
+	 * @param precision
+	 * @return
+	 */
+	public abstract LispString toLispString(int radix, int precision);
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract LispReal norm();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract LispNumber conjugate();
+
 	/**
 	 * 
 	 * @return
@@ -1118,8 +1335,7 @@ public abstract class LispNumber extends Atom {
 	 */
 	public abstract int getInt();
 
-	/*
-	 * (non-Javadoc)
+	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Datum#getLong()
 	 */
 	public abstract long getLong();
@@ -1130,18 +1346,42 @@ public abstract class LispNumber extends Atom {
 	 */
 	public abstract BigInteger getBigInteger();
 
-	/*
-	 * (non-Javadoc)
+	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Datum#getBigDecimal()
 	 */
 	public abstract BigDecimal getBigDecimal();
 
-	/*
-	 * (non-Javadoc)
+	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Datum#getRealDouble()
 	 */
 	public abstract double getRealDouble();
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.Datum#getRealFloat()
+	 */
+	public float getRealFloat() {
+		return (float)getRealDouble();
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.Datum#getRealHalf()
+	 */
+	public short getRealHalf() {
+		return DoubleUtils.toHalf(getRealDouble());
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.Datum#getRealDouble()
+	 */
+	public abstract long getRealDecimal64();
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.Datum#getRealDecimal32()
+	 */
+	public int getRealDecimal32() {
+		return Decimal32.decimal64To32(getRealDecimal64());
+	}
+
 	/**
 	 * 
 	 * @return
@@ -1154,6 +1394,69 @@ public abstract class LispNumber extends Atom {
 	 */
 	public abstract BigInteger getDenominator();
 
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract LispReal[] getImags();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract double[] getImagsDouble();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract LispReal[] getImagsAsQuaternion();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract double[] getImagsDoubleAsQuaternion();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract LispReal[] getImagsAsOctonion();
+
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract double[] getImagsDoubleAsOctonion();
+
+	/**
+	 * 
+	 * @param n
+	 * @return
+	 */
+	public LispNumber pow(int n) {
+		LispNumber r = LispInteger.ONE;
+
+		if(n >= 0) {
+			for(int i = 0; i < n; i++) {
+				r = r.mul(this);
+			}
+		} else {
+			for(int i = 0; i < -n; i++) {
+				r = r.div(this);
+			}
+		}
+		return r;
+	}
+
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.math.algebra.ILispInvertable#invert()
+	 */
+	public LispNumber inv() {
+		return LispInteger.ONE.div(this);
+	}
+
 	/**
 	 * 
 	 * @return
@@ -1178,6 +1481,12 @@ public abstract class LispNumber extends Atom {
 		return isEqualTo(LispInteger.ZERO);
 	}
 
+	/**
+	 * 
+	 * @return
+	 */
+	public abstract boolean isFinite();
+
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.Datum#getType()
 	 */
