@@ -15,6 +15,8 @@
  */
 package net.morilib.lisp;
 
+import net.morilib.lisp.condition.LispSimpleCondition;
+
 public class ReadFileException extends LispException {
 
 	//
@@ -33,21 +35,24 @@ public class ReadFileException extends LispException {
 	 * @param cause
 	 */
 	public ReadFileException(String message, Throwable cause) {
-		super(ERR_CODE, message, cause);
+		super(ERR_CODE, message,
+				LispSimpleCondition.newInstance("&violation"), cause);
 	}
 
 	/**
 	 * @param message
 	 */
 	public ReadFileException(String message) {
-		super(ERR_CODE, message);
+		super(ERR_CODE, message,
+				LispSimpleCondition.newInstance("&violation"));
 	}
 
 	/**
 	 * @param cause
 	 */
 	public ReadFileException(Throwable cause) {
-		super(ERR_CODE, cause);
+		super(ERR_CODE,
+				LispSimpleCondition.newInstance("&violation"), cause);
 	}
 
 }
