@@ -17,35 +17,55 @@ package net.morilib.lisp;
 
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2009
+ */
 /*package*/ final class SyntaxUtils {
-	
+
+	//
+	private static final Symbol SYNTAX_RULES =
+		Symbol.getSymbol("syntax-rules");
+
+	//
+	private static final Symbol IDENTIFIER_SYNTAX =
+		Symbol.getSymbol("identifier-syntax");
+
+	//
+	private static final Symbol SETBANG = Symbol.getSymbol("set!");
+
+	//
 	private SyntaxUtils() {
 		// do nothing
 	}
-	
-	
+
+	//
 	/*package*/ static interface SafeWrap {
-		
+
+		//
 		public Datum getWrapee();
-		
+
 	}
-	
-	
+
+	//
 	/*package*/ static boolean isValidSymbolList(Datum d) {
 		Datum p = d;
-		
+
 		if(d.isTypeSymbol()) {
 			return true;
 		}
-		
+
 		while(p != Nil.NIL) {
 			if(p instanceof Cons) {
 				Cons c = (Cons)p;
-				
+
 				if(!c.getCar().isTypeSymbol()) {
 					return false;
 				}
@@ -56,8 +76,8 @@ import java.util.Set;
 		}
 		return true;
 	}
-	
-	
+
+	//
 	/*package*/ static void compileBind(
 			Datum sym,
 			Datum bcdr,
@@ -74,30 +94,30 @@ import java.util.Set;
 			LispCompiler.MiscInfo syncased) {
 		if(bcdr instanceof Cons) {
 			Datum d1 = ((Cons)bcdr).getCar();
-			
+
 			// create a temporary Closure
 			CompiledCode.Builder nbuild = new CompiledCode.Builder();
 			Environment  nenv = new Environment(env);
-			
+
 			comp.compile(
 					d1, nenv, nbuild, callsym, true,
 					new ArrayList<Cons>(),
 					exec, memento, syncased);
 			nbuild.addReturnOp();
-			
-			// 一時的なClosureを生成する
+
+			// create a temporary Closure
 			ClosureClass cl = new ClosureClass(
 					Nil.NIL, nbuild.getCodeRef());
-			
-			// 一時的なClosureを呼び出す
+
+			// call the temporary Closure
 			build.addPush(cl);
 			build.addBeginList();
 			build.addEndList();
 			build.addCall();
-			
+
 			if(toplevel) {
 				Symbol r;
-				
+
 				//if(sym instanceof Symbol) {
 				//	r = ((Symbol)sym).getEnclosedSymbol();
 				//} else {
@@ -124,8 +144,8 @@ import java.util.Set;
 			throw mesg.getError(ercd);
 		}
 	}
-	
-	
+
+	//
 	/*package*/ static void compileBindMacro(
 			Datum sym,
 			Datum bcdr,
@@ -141,27 +161,27 @@ import java.util.Set;
 			LispCompiler.MiscInfo syncased) {
 		if(bcdr instanceof Cons) {
 			Datum d1 = ((Cons)bcdr).getCar();
-			
+
 			// create a temporary Closure
 			CompiledCode.Builder nbuild = new CompiledCode.Builder();
 			Environment  nenv = new Environment(env);
-			
+
 			comp.compile(
 					d1, nenv, build, callsym, true,
 					new ArrayList<Cons>(),
 					exec, memento, syncased);
 			nbuild.addReturnOp();
-			
-			// 一時的なClosureを生成する
+
+			// create a temporary Closure
 			ClosureClass cl = new ClosureClass(
 					Nil.NIL, nbuild.getCodeRef());
-			
-			// 一時的なClosureを呼び出す
+
+			// call the temporary Closure
 			build.addPush(cl);
 			build.addBeginList();
 			build.addEndList();
 			build.addCall();
-			
+
 			build.addBindMacro(sym);
 			//build.addPush(bcar);
 			build.addPush(Undef.UNDEF);
@@ -170,8 +190,100 @@ import java.util.Set;
 			throw mesg.getError(ercd);
 		}
 	}
-	
-	
+
+	//
+	/*package*/ static void compileBindMacro72(
+			Datum sym,
+			Datum bcdr,
+			Environment env,
+			LispCompiler comp,
+			CompiledCode.Builder build,
+			Cons callsym,
+			LispMessage mesg,
+			String ercd,
+			List<Cons> symlist,
+			CodeExecutor exec,
+			IntStack memento,
+			LispCompiler.MiscInfo syncased) {
+		if(bcdr instanceof Cons) {
+			Datum d1 = ((Cons)bcdr).getCar();
+
+			// create a temporary Closure
+			CompiledCode.Builder nbuild = new CompiledCode.Builder();
+			Environment  nenv = new Environment(env);
+
+			comp.compile(
+					d1, nenv, build, callsym, true,
+					new ArrayList<Cons>(),
+					exec, memento, syncased);
+			nbuild.addReturnOp();
+
+			// create a temporary Closure
+			ClosureClass cl = new ClosureClass(
+					Nil.NIL, nbuild.getCodeRef());
+
+			// call the temporary Closure
+			build.addPush(cl);
+			build.addBeginList();
+			build.addEndList();
+			build.addCall();
+
+			build.addBindMacro72(sym);
+			//build.addPush(bcar);
+			build.addPush(Undef.UNDEF);
+		} else {
+			//throw new LispException("Syntax error");
+			throw mesg.getError(ercd);
+		}
+	}
+
+	//
+	/*package*/ static void compileBindMacroQuote(
+			Datum sym,
+			Datum bcdr,
+			Environment env,
+			LispCompiler comp,
+			CompiledCode.Builder build,
+			Cons callsym,
+			LispMessage mesg,
+			String ercd,
+			List<Cons> symlist,
+			CodeExecutor exec,
+			IntStack memento,
+			LispCompiler.MiscInfo syncased) {
+		if(bcdr instanceof Cons) {
+			Datum d1 = ((Cons)bcdr).getCar();
+
+			// create a temporary Closure
+			CompiledCode.Builder nbuild = new CompiledCode.Builder();
+			Environment  nenv = new Environment(env);
+
+			comp.compile(
+					d1, nenv, build, callsym, true,
+					new ArrayList<Cons>(),
+					exec, memento, syncased);
+			nbuild.addReturnOp();
+
+			// create a temporary Closure
+			ClosureClass cl = new ClosureClass(
+					Nil.NIL, nbuild.getCodeRef());
+
+			// call the temporary Closure
+			build.addPush(cl);
+			build.addBeginList();
+			build.addEndList();
+			build.addCall();
+
+			build.addBindMacroQuote(sym);
+			//build.addPush(bcar);
+			build.addPush(Undef.UNDEF);
+		} else {
+			//throw new LispException("Syntax error");
+			throw mesg.getError(ercd);
+		}
+	}
+
+	//
 	/*package*/ static void compileList(
 			Datum b,
 			Environment env,
@@ -186,29 +298,27 @@ import java.util.Set;
 			boolean toplv,
 			LispCompiler.MiscInfo syncased) {
 		Datum bcdr = b;
-		
+
 		nbuild.addPush(Undef.UNDEF);
 		while(bcdr != Nil.NIL) {
 			if(bcdr instanceof Cons) {
 				Cons bc = (Cons)bcdr;
 				boolean tl = istail && (bc.getCdr() == Nil.NIL);
-				
-				// 前の値を捨てる
+
+				// dispose previous value
 				nbuild.addPop();
-				
 				comp.compile(
 						bc.getCar(), env, nbuild, toplv,
 						callsym, tl, symlist,
 						exec, memento, syncased);
 				bcdr = bc.getCdr();
 			} else {
-				//throw new LispException("proper list required");
 				throw mesg.getError("err.explist.malform");
 			}
 		}
 	}
-	
-	
+
+	//
 	/*package*/ static void compileList(
 			Datum b,
 			Environment env,
@@ -227,8 +337,8 @@ import java.util.Set;
 				symlist, exec, memento, false,
 				syncased);
 	}
-	
-	
+
+	//
 	/*package*/ static void compileListApply(
 			Datum b,
 			Environment env,
@@ -241,11 +351,11 @@ import java.util.Set;
 			IntStack memento,
 			LispCompiler.MiscInfo syncased) {
 		Datum bcdr = b;
-		
+
 		while(bcdr != Nil.NIL) {
 			if(bcdr instanceof Cons) {
 				Cons bc = (Cons)bcdr;
-				
+
 				comp.compile(
 						bc.getCar(),
 						env, nbuild, callsym, false, symlist,
@@ -253,13 +363,12 @@ import java.util.Set;
 				nbuild.addAppendList();
 				bcdr = bc.getCdr();
 			} else {
-				//throw new LispException("proper list required");
 				throw mesg.getError("err.explist.malform");
 			}
 		}
 	}
-	
-	
+
+	//
 	/*package*/ static Datum replaceLocalValsList(
 			Datum b,
 			Environment env,
@@ -270,7 +379,7 @@ import java.util.Set;
 			int ttype) {
 		Datum bcdr = b;
 		List<Datum> lst = new ArrayList<Datum>();
-		
+
 		while(bcdr != Nil.NIL) {
 			if(bcdr instanceof Cons) {
 				lst.add(comp.replaceLocalVals(
@@ -278,15 +387,14 @@ import java.util.Set;
 						env, ienv, toplv, ttype));
 				bcdr = ((Cons)bcdr).getCdr();
 			} else {
-				//throw new LispException("proper list required");
 				throw mesg.getError("err.explist.malform");
 			}
 		}
-		
+
 		return LispUtils.listToCons(lst);
 	}
-	
-	
+
+	//
 	/*package*/ static Datum replaceLocalValsList(
 			Datum b,
 			Environment env,
@@ -297,14 +405,14 @@ import java.util.Set;
 		return replaceLocalValsList(
 				b, env, comp, ienv, mesg, false, ttype);
 	}
-	
-	
+
+	//
 	/*package*/ static Datum putSymbol(
 			Environment ienv, Datum p, LispMessage mesg) {
 		if(p instanceof Symbol) {
 			//Symbol res = Symbol.gensym();
 			Symbol res = Symbol.encloseSymbol((Symbol)p);
-			
+
 			ienv.bindDatum(p, res);
 			return res;
 		} else if(p instanceof SymbolScope) {
@@ -312,7 +420,7 @@ import java.util.Set;
 			//Symbol res = Symbol.gensym();
 			//Symbol res = Symbol.encloseSymbol(s);
 			Symbol res;
-			
+
 			res = ((SymbolScope)p).isCaptured() ?
 					s : Symbol.encloseSymbol(s);
 			ienv.bindDatum(s, res);
@@ -321,22 +429,21 @@ import java.util.Set;
 			return p;
 		} else {
 			//System.out.println(p);
-			//throw new LispException("arguments must be a symbol");
 			throw mesg.getError("err.symbol");
 		}
 	}
-	
-	
+
+	//
 	/*package*/ static Datum addLocalValsAll(
 			Environment ienv, Datum d, LispMessage mesg) {
 		Datum p = d;
 		List<Datum> lst = new ArrayList<Datum>();
-		
+
 		while(p != Nil.NIL) {
 			if(p instanceof Cons) {
 				Datum r = putSymbol(ienv, ((Cons)p).getCar(), mesg);
 				p = ((Cons)p).getCdr();
-				
+
 				lst.add(r);
 			} else {
 				return LispUtils.listToCons(
@@ -346,12 +453,8 @@ import java.util.Set;
 		}
 		return LispUtils.listToCons(lst);
 	}
-	
-	
+
 	//
-	private static final Symbol SYNTAX_RULES =
-		Symbol.getSymbol("syntax-rules");
-	
 	private static void processRule(
 			Symbol name,
 			Datum body,
@@ -366,13 +469,12 @@ import java.util.Set;
 		PatternDepthMap mpp = new PatternDepthMap();
 		PatternDepthMap mpt = new PatternDepthMap();
 		Datum ptd;
-		
+
 		if(lst.size() != 2) {
-			//throw new LispException("malformed syntax-rules");
 			throw mesg.getError("err.syntaxrules.malform");
 		}
-		
-		// パターンを取得
+
+		// get a pattern
 		//if(lst.get(0) instanceof Cons) {
 		//	Cons l0 = (Cons)lst.get(0);
 		//	
@@ -399,7 +501,7 @@ import java.util.Set;
 			throw mesg.getError("err.wrongellipsis");
 		}
 		pat.add(ptd);
-		
+
 		// validate the template
 		Datum tpl = PatternMatch.compileTemplate(lst.get(1), mpt);
 		try {
@@ -407,14 +509,15 @@ import java.util.Set;
 		} catch (PatternDepthException e) {
 			throw mesg.getError("err.wronglevel", e.getMessage());
 		}
-		
-		// テンプレートを取得
+
+		// get a template
 		tmp.add(lst.get(1));
-		
-		// 繰り返し文字列の集合を格納
+
+		// add a set of repeating strings
 		stl.add(st);
 	}
-	
+
+	//
 	/*package*/ static UserSyntax processSyntaxRules(
 			Symbol name,
 			Datum body,
@@ -422,32 +525,30 @@ import java.util.Set;
 			LispMessage mesg,
 			UserSyntax rootsyn) {
 		List<Datum> lst = LispUtils.consToList(body, mesg);
-		
+
 		if(lst.size() < 2) {
-			//throw new LispException("malformed syntax-rules");
 			throw mesg.getError("err.syntaxrules.malform");
 		}
-		
-		// 予約語を取得
+
+		// get reserved words
 		Set<Symbol> reserve =
 			SyntaxUtils.consToSymbolSet(lst.get(0), mesg);
-		
-		// ルールを取得
+
+		// get rules
 		List<Datum> pat = new ArrayList<Datum>();
 		List<Datum> tmp = new ArrayList<Datum>();
 		List<Set<Symbol>> stl = new ArrayList<Set<Symbol>>();
-		
+
 		for(int i = 1; i < lst.size(); i++) {
 			processRule(
 					name, lst.get(i), pat, tmp, stl, reserve, mesg);
 		}
-		
 		return new UserSyntax(
 				name.getName(), pat, tmp, stl, reserve, env,
 				rootsyn);
 	}
-	
-	
+
+	//
 	/*package*/ static UserSyntax processRuleDesc(
 			Symbol name,
 			Datum body,
@@ -455,63 +556,66 @@ import java.util.Set;
 			LispMessage mesg,
 			UserSyntax rootsyn) {
 		if(!(body instanceof Cons)) {
-			//throw new LispException("malformed syntax-rules");
 			throw mesg.getError("err.syntaxrules.malform");
 		}
-		
+
 		Cons b1 = (Cons)body;
-		
+
 		if(SYNTAX_RULES.equals(b1.getCar())) {
 			return processSyntaxRules(
 					name, b1.getCdr(), env, mesg, rootsyn);
 		} else {
-			//throw new LispException("malformed syntax-rules");
 			throw mesg.getError("err.syntaxrules.malform");
 		}
 	}
-	
-	
+
+	//
 	/*package*/ static boolean isSyntaxRules(Datum body) {
 		if(!(body instanceof Cons)) {
 			return false;
+		} else {
+			return SYNTAX_RULES.equals(((Cons)body).getCar());
 		}
-		
-		Cons b1 = (Cons)body;
-		
-		return SYNTAX_RULES.equals(b1.getCar());
 	}
-	
 
+	//
+	/*package*/ static boolean isIdentifierSyntax(Datum body) {
+		if(!(body instanceof Cons)) {
+			return false;
+		} else {
+			return IDENTIFIER_SYNTAX.equals(((Cons)body).getCar());
+		}
+	}
+
+	//
 	/*package*/ static Set<Symbol> consToSymbolSet(
 			Datum d, LispMessage mesg) {
 		Set<Symbol> res = new HashSet<Symbol>();
 		Datum dd = d;
-		
+
 		while(dd != Nil.NIL) {
 			if(dd instanceof Cons) {
 				Cons cd = (Cons)dd;
-				
+
 				if(cd.getCar() instanceof SymbolName) {
 					Symbol smb = ((SymbolName)cd.getCar()).getSymbol();
-					
+
 					//res.add((Symbol)cd.getCar());
 					res.add(smb);
 				} else {
-					//throw new LispException("must be a symbol list");
 					throw mesg.getError("err.list.symbol");
 				}
-				
+
 				dd = ((Cons)dd).getCdr();
 			} else {
-				//throw new NotProperException("proper list required");
 				throw mesg.getError("err.list");
 			}
 		}
-		
+
 		return res;
 	}
-	
-	
+
+	//
 	/*package*/ static boolean equalsReserved(Symbol rword, Datum d) {
 		if(d instanceof SymbolName) {
 			Symbol s1 = ((SymbolName)d).getSymbol();
@@ -522,30 +626,31 @@ import java.util.Set;
 			return rword.equals(d);
 		}
 	}
-	
+
 	//
 	/*package*/ static Datum removeScope1(
 			Datum d, Collection<Datum> col) {
 		if(d instanceof SymbolName) {
 			Symbol s = ((SymbolName)d).getSymbol();
-			
+
 			if(col.contains(s)) {
 				return s;
 			}
 		}
 		return d;
 	}
-	
+
+	//
 	/*package*/ static Datum removeScope(
 			Datum body, Collection<Datum> col) {
 		ConsIterator    itr = new ConsIterator(body);
-		
+
 		if(itr.hasNext()) {
 			ConsListBuilder bld = new ConsListBuilder();
-			
+
 			while(itr.hasNext()) {
 				Datum c = itr.next();
-				
+
 				bld.append(removeScope(c, col));
 			}
 			return bld.get(removeScope1(itr.getTerminal(), col));
@@ -553,5 +658,76 @@ import java.util.Set;
 			return removeScope1(itr.getTerminal(), col);
 		}
 	}
-	
+
+	//
+	private static Datum _caaaddr(Datum d) {
+		Datum e = d;
+
+		if(!(e instanceof Cons)) {
+			return Undef.UNDEF;
+		} else if(!((e = ((Cons)e).getCdr()) instanceof Cons)) {
+			return Undef.UNDEF;
+		} else if(!((e = ((Cons)e).getCdr()) instanceof Cons)) {
+			return Undef.UNDEF;
+		} else if(!((e = ((Cons)e).getCar()) instanceof Cons)) {
+			return Undef.UNDEF;
+		} else if(!((e = ((Cons)e).getCar()) instanceof Cons)) {
+			return Undef.UNDEF;
+		} else {
+			return ((Cons)e).getCar();
+		}
+	}
+
+	/**
+	 * @param name
+	 * @param car
+	 * @param mesg
+	 * @return
+	 */
+	public static Datum processIdentifierSyntax(Symbol name, Datum car,
+			Environment env, LispMessage mesg) {
+		UserSyntax syn;
+		Datum ptd, ptn, tmp, tpl;
+
+		if(_caaaddr(car).equals(SETBANG)) {
+			Set<Symbol> res = Collections.emptySet();
+			Set<Symbol> st  = new HashSet<Symbol>();
+			PatternDepthMap mpp = new PatternDepthMap();
+			PatternDepthMap mpt = new PatternDepthMap();
+
+			tmp = IntLispUtils.cadaddr(car, mesg);
+			ptn = IntLispUtils.caddaaddr(car, mesg);
+			try {
+				ptd = PatternMatch.compilePattern(ptn, mpp, st, res);
+			} catch (PatternEllipsisException e) {
+				throw mesg.getError("err.wrongellipsis");
+			}
+
+			// validate the template
+			tpl = PatternMatch.compileTemplate(tmp, mpt);
+			try {
+				PatternMatch.validateLevel(ptd, tpl, st);
+			} catch (PatternDepthException e) {
+				throw mesg.getError("err.wronglevel", e.getMessage());
+			}
+
+			syn = new UserSyntax(
+					name.getName(),
+					Collections.singletonList(ptd),
+					Collections.singletonList(tpl),
+					Collections.singletonList(st),
+					res,
+					env,
+					null);
+			return new UserIdentifierSyntax(
+					name.getName(),
+					IntLispUtils.cadadr(car, mesg),
+					syn);
+		} else {
+			return new UserIdentifierSyntax(
+					name.getName(),
+					IntLispUtils.cadr(car, mesg));
+		}
+	}
+
 }
