@@ -89,7 +89,7 @@ implements java.io.Serializable {
 		/* (non-Javadoc)
 		 * @see net.morilib.lisp.array.LispArrayPrototype#makeArray(int[])
 		 */
-		public LispArray makeArray(int... is) {
+		public ILispArray makeArray(int... is) {
 			return new LispArrayU8(real, is);
 		}
 
@@ -177,7 +177,7 @@ implements java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isEqualTo(LispArray a) {
+	public boolean isEqualTo(ILispArray a) {
 		if(!isIndexEqualTo(a)) {
 			return false;
 		} else if(a instanceof LispArrayU8) {
