@@ -84,7 +84,7 @@ implements java.io.Serializable {
 		/* (non-Javadoc)
 		 * @see net.morilib.lisp.array.LispArrayPrototype#makeArray(int[])
 		 */
-		public LispArray makeArray(int... is) {
+		public ILispArray makeArray(int... is) {
 			return malloc(is, datum);
 		}
 
@@ -190,7 +190,7 @@ implements java.io.Serializable {
 	 * 
 	 * @param itr
 	 */
-	public static boolean isEqualTo(LispArray a, LispArray b) {
+	public static boolean isEqualTo(ILispArray a, ILispArray b) {
 		int[] is;
 
 		if(!a.isIndexEqualTo(b)) {
@@ -340,7 +340,7 @@ implements java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isIndexEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isIndexEqualTo(LispArray a) {
+	public boolean isIndexEqualTo(ILispArray a) {
 		if(rank() != a.rank()) {
 			return false;
 		} else {
@@ -358,7 +358,7 @@ implements java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isEqualTo(LispArray a) {
+	public boolean isEqualTo(ILispArray a) {
 		return isEqualTo(this, a);
 	}
 
