@@ -88,14 +88,14 @@ implements java.io.Serializable {
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isIndexEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isIndexEqualTo(LispArray a) {
+	public boolean isIndexEqualTo(ILispArray a) {
 		return a.rank() == 0;
 	}
 
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.array.LispArray#isEqualTo(net.morilib.lisp.array.LispArray)
 	 */
-	public boolean isEqualTo(LispArray a) {
+	public boolean isEqualTo(ILispArray a) {
 		if(a.rank() == 0) {
 			return LispUtils.equals(datum, a.getFromArray());
 		}
