@@ -1349,6 +1349,5 @@
 
 (cond
   ((not (defined? srfi-43))
-    (import (srfi-43))
-    (format #t "~a~%" srfi-43)))
+    (import (srfi-43))))
 ;; END
