@@ -37,7 +37,7 @@ public class StringDrop extends BinaryArgs {
 			Datum c1a, Datum c2a, Environment env,
 			LispMessage mesg) {
 		String s = SubrUtils.getString(c1a, mesg);
-		int nc = SubrUtils.getSmallIntegerExact(c2a, mesg);
+		int nc = SubrUtils.getSmallInt(c2a, mesg);
 
 		if(nc > s.length()) {
 			throw mesg.getError("err.string.outofrange", c2a);
