@@ -19,7 +19,7 @@ import java.nio.ReadOnlyBufferException;
 
 import net.morilib.lisp.Datum;
 import net.morilib.lisp.Environment;
-import net.morilib.lisp.LispEndianness;
+import net.morilib.lisp.SRFI74Endianness;
 import net.morilib.lisp.LispInteger;
 import net.morilib.lisp.LispMessage;
 import net.morilib.lisp.Undef;
@@ -50,13 +50,13 @@ public class BlobS16SetS extends QuaternaryArgs {
 	@Override
 	protected Datum execute(Datum c1a, Datum c2a, Datum c3a, Datum c4a,
 			Environment env, LispMessage mesg) {
-		if(!(c1a instanceof LispEndianness.Endian)) {
+		if(!(c1a instanceof SRFI74Endianness.Endian)) {
 			throw mesg.getError("err.srfi74.require.endianness", c1a);
 		} else if(c2a instanceof LispBlob) {
 			int k = SubrUtils.getSmallInt(c3a, mesg);
 
 			try {
-				((LispBlob)c2a).order((LispEndianness.Endian)c2a);
+				((LispBlob)c2a).order((SRFI74Endianness.Endian)c2a);
 				((LispBlob)c2a).putShort(k, getNumber(c4a, mesg));
 				return Undef.UNDEF;
 			} catch(IndexOutOfBoundsException e) {
