@@ -17,7 +17,7 @@ package net.morilib.lisp.nio;
 
 import net.morilib.lisp.Datum;
 import net.morilib.lisp.Environment;
-import net.morilib.lisp.LispEndianness;
+import net.morilib.lisp.SRFI74Endianness;
 import net.morilib.lisp.LispInteger;
 import net.morilib.lisp.LispMessage;
 import net.morilib.lisp.subr.SubrUtils;
@@ -37,13 +37,13 @@ public class BlobU64Ref extends TernaryArgs {
 	@Override
 	protected Datum execute(Datum c1a, Datum c2a, Datum c3a,
 			Environment env, LispMessage mesg) {
-		if(!(c1a instanceof LispEndianness.Endian)) {
+		if(!(c1a instanceof SRFI74Endianness.Endian)) {
 			throw mesg.getError("err.srfi74.require.endianness", c1a);
 		} else if(c2a instanceof LispBlob) {
 			int k = SubrUtils.getSmallInt(c3a, mesg);
 
 			try {
-				((LispBlob)c1a).order((LispEndianness.Endian)c2a);
+				((LispBlob)c1a).order((SRFI74Endianness.Endian)c2a);
 				return LispInteger.valueOf(Bytes.ulongToBigInteger(
 						((LispBlob)c2a).getInt(k)));
 			} catch(IndexOutOfBoundsException e) {
