@@ -17,19 +17,36 @@ package net.morilib.lisp.subr;
 
 import net.morilib.lisp.LispCharacter;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2009
+ */
 public class CharCIGreaterThanEq extends CharCompare {
 
+	/*
+	 * (non-Javadoc)
+	 * @see net.morilib.lisp.subr.CharCompare#compare(net.morilib.lisp.LispCharacter, net.morilib.lisp.LispCharacter)
+	 */
 	@Override
 	protected boolean compare(LispCharacter c1a, LispCharacter c2a) {
-		char ch1 = Character.toUpperCase(c1a.getCharacter());
-		char ch2 = Character.toUpperCase(c2a.getCharacter());
-		
+//		char ch1 = Character.toUpperCase(c1a.getCharacter());
+//		char ch2 = Character.toUpperCase(c2a.getCharacter());
+		int ch1 = SubrUtils.toFoldCase(c1a.getCharacterCodePoint());
+		int ch2 = SubrUtils.toFoldCase(c2a.getCharacterCodePoint());
+
+//		ch1 = Character.toLowerCase(ch1);
+//		ch2 = Character.toLowerCase(ch2);
 		return ch1 >= ch2;
 	}
-	
-	
+
+	/*
+	 * (non-Javadoc)
+	 * @see net.morilib.lisp.Subr#toString()
+	 */
 	public String toString() {
 		return "Subr:char-ci>=?";
 	}
-	
+
 }
