@@ -19,6 +19,7 @@ import net.morilib.lang.system.Version;
 import net.morilib.lisp.Datum;
 import net.morilib.lisp.Environment;
 import net.morilib.lisp.LispMessage;
+import net.morilib.lisp.Scheme;
 import net.morilib.lisp.subr.NoArgs;
 
 /**
@@ -32,7 +33,7 @@ public class SchemeVersion extends NoArgs {
 	 * 
 	 */
 	public static final LispVersion VERSION =
-		new LispVersion(new Version("0.4.0"));
+		new LispVersion(new Version(Scheme.SCHLUESSEL_VERSION));
 
 	/* (non-Javadoc)
 	 * @see net.morilib.lisp.subr.NoArgs#execute(net.morilib.lisp.Environment, net.morilib.lisp.LispMessage)
