@@ -24,13 +24,17 @@ import net.morilib.util.primitive.iterator.LongVectorIterator;
  * @author MORIGUCHI, Yuichiro 2010/10/11
  */
 public interface LongVector extends List<Long>, LongList {
-	
+
 	public int lastIndexOfLong(long v);
-	
+
 	public LongVectorIterator longVectorIterator();
-	
+
 	public LongVectorIterator longVectorIterator(int index);
-	
+
 	public LongVector subVector(int start, int end);
-	
+
+	public boolean addAllLong(long[] b);
+
+	public boolean addAllLong(int index, long[] b);
+
 }
